//====----------------------- --- --   -      -
// MULTIUSER NETWORK MANAGEMENT
// TCP/IP and UDP support.
// 2000-2002, Vander Nunes
//======----------------------------- --- --   -      -

#ifndef _NETCLIENT_H_
#define _NETCLIENT_H_

  #include <time.h>
  #include "net.h"


  // ----------------------------------------------------------------------------------------------

  // user's callback
  #define USR_CLNTCALLBACK(userProc) void (*userProc)(CNetClient *pClient, void *pUserData, int iMsg)


  // ----------------------------------------------------------------------------------------------
  // NetClient class
  // ----------------------------------------------------------------------------------------------
  class CNetClient
  {
    // internal window procedure
    friend LRESULT CALLBACK NetClientWndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam);

    private:

      // reference count
      static DWORD        m_dwRefCount;

      // netclient hidden window
      HWND                m_hWnd;

      // connection information
      SOCKET              m_Sock;
      enSockType          m_SockType;
      int                 m_iPort;

      // receive buffer
      char*               m_pRecvBuffer;
      int                 m_iRecvBufferSize;

      // a network packet
      NETPACKET           m_Packet;

      // client ip address
      struct sockaddr_in  m_MyAddr;
      
      // host ip address
      struct sockaddr_in  m_HostAddr;

      // flag if the class is correctly initialized
      BOOL                m_bInitialized;

      // user callback procedure
      USR_CLNTCALLBACK(m_userProc);
      void*               m_pUserData;

      // used by the non-blocking gethostbyname()
      HANDLE              m_hResolver;
      char                m_szHostData[MAXGETHOSTSTRUCT];

      // ------------------------------------------------------------------------------------------
      // private methods
      // ------------------------------------------------------------------------------------------

      BOOL        Init(enSockType SockType, USR_CLNTCALLBACK(userProc), void *pUserData, int iBufferSize);
      BOOL        Select(int wParam, int lParam);

      // after resolving the host, this will
      // be called to continue connection process
      BOOL CNetClient::ContinueConnect(struct hostent *phe);

    public:

      // ------------------------------------------------------------------------------------------
      // constructor - user must provide a callback function address to receive network events.
      // a NULL callback address is accepted but in this case the user can't receive warnings
      // about connection, disconnection, new packets, etc.
      // optionally can be passed a pointer to any desired user data. this pointer will be passed
      // back in any call to the user's callback.
      // ------------------------------------------------------------------------------------------

      CNetClient(enSockType SockType, USR_CLNTCALLBACK(userProc), void *pUserData, int iBufferSize=1024);

      // ------------------------------------------------------------------------------------------
      // destructor - closes connection, clean resources.
      // ------------------------------------------------------------------------------------------

      virtual ~CNetClient();

      // ------------------------------------------------------------------------------------------
      // connect to a hostname through specified port.
      // ------------------------------------------------------------------------------------------

      BOOL        Connect(char *szHost, int iPort);

      // ------------------------------------------------------------------------------------------
      // disconnects from actual host.
      // ------------------------------------------------------------------------------------------

      void        Disconnect();

      // ------------------------------------------------------------------------------------------
      // read and return a pointer to a network packet.
      // ------------------------------------------------------------------------------------------

      NETPACKET*  Read();

      // ------------------------------------------------------------------------------------------
      // send data to the actual host.
      // ------------------------------------------------------------------------------------------

      void        Send(void *Msg, int MsgLen);
  };

#endif
