//====----------------------- --- --   -      -
// MULTIUSER NETWORK MANAGEMENT
// TCP/IP and UDP support.
// 2000-2002, Vander Nunes
//======----------------------------- --- --   -      -


#ifndef _NETSERVER_H_
#define _NETSERVER_H_

  #include <time.h>
  #include "net.h"


  // ----------------------------------------------------------------------------------------------

  // user's callback
  #define USR_SRVRCALLBACK(userProc) void (*userProc)(CNetServer *pServer, void *pUserData, int iMsg, int iClientIdx)

  // connection information
  typedef struct _conn
  {
    struct sockaddr_in rem_addr;    // net address of other end
    SOCKET sock;                    // socket(s) allocated for net traffic, waiting for a response.
    int    protocol;                // protocol to use when calling client
    long   lasttick;                // last tick when we heard something from this client
    char   used;                    // 0 = empty slot
  } CONN_INFO;


  // ----------------------------------------------------------------------------------------------
  // NetServer class
  // ----------------------------------------------------------------------------------------------
  class CNetServer
  {
    // internal window procedure
    friend LRESULT CALLBACK NetWndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam);

    private:

      // reference count
      static DWORD        m_dwRefCount;

      // netserver hidden window
      HWND                m_hWnd;

      enSockType          m_SockType;
      int                 m_iListenPort;
      int                 m_iMaxClients;
      CONN_INFO*          m_pClients;
      int                 m_iNumClients;

      char*               m_pRecvBuffer;
      int                 m_iRecvBufferSize;
      NETPACKET           m_Packet;

      SOCKET              m_UDPInSock;
      SOCKET              m_TCPInSock;
      BOOL                m_bInitialized;

      // default timeout to auto-disconnect clients.
      // this is useful to handle UDP connections too.
      // ### because my time constraints, I didn't finish Timeout handling yet.
      int                 m_iTimeout;

      // server ip address
      struct sockaddr_in  m_MyAddr;

      // user callback procedure
      USR_SRVRCALLBACK(m_userProc);
      void               *m_pUserData;


      // ------------------------------------------------------------------------------------------
      // initialize winsock
      // ------------------------------------------------------------------------------------------

      BOOL        Init(int iMaxClients, int iBufferSize);
      BOOL        Select(int wParam, int lParam);
      BOOL        NewConnection(SOCKET ServerSocket);
      int         FindClientIdx(SOCKET Socket);

    public:

      CNetServer(enSockType SockType, USR_SRVRCALLBACK(userProc), void *pUserData, int iListenPort=1970, int iMaxClients=32, int iBufferSize=1024, int iTimeout=60000);
      virtual ~CNetServer();

      BOOL        Listen(enSockType SockType, USR_SRVRCALLBACK(userProc), void *pUserData, int iListenPort, int iTimeout=60000);
      void        StopListening();

      NETPACKET*  ReadMessageFromClient(int iClientIdx);
      void        SendMessageToClient(int ClientIdx, void *Msg, int MsgLen);
      void        BroadcastMessage(void *Msg, int MsgLen);

      CONN_INFO*  Client(int iClientIdx);

  };

#endif
